@@ -17,7 +17,7 @@ https://www.python.org/dev/peps/pep-0249/ .
 Fetch methods returns rows as a list of lists on purpose to let the caller
 decide to convert then to a list of tuples.
 """
-
+from decimal import Decimal
 from typing import Any, List, Optional  # NOQA for mypy types
 
 import copy
@@ -196,7 +196,7 @@ class Connection(object):
             self.request_timeout,
         )
 
-    def cursor(self):
+    def cursor(self, experimental_python_types=False):
         """Return a new :py:class:`Cursor` object using the connection."""
         if self.isolation_level != IsolationLevel.AUTOCOMMIT:
             if self.transaction is None:
@@ -206,7 +206,7 @@ class Connection(object):
             request = self.transaction._request
         else:
             request = self._create_request()
-        return Cursor(self, request)
+        return Cursor(self, request, experimental_python_types)
 
 
 class Cursor(object):
@@ -217,7 +217,7 @@ class Cursor(object):
 
     """
 
-    def __init__(self, connection, request):
+    def __init__(self, connection, request, experimental_python_types: bool = False):
         if not isinstance(connection, Connection):
             raise ValueError(
                 "connection must be a Connection object: {}".format(type(connection))
@@ -228,6 +228,7 @@ class Cursor(object):
         self.arraysize = 1
         self._iterator = None
         self._query = None
+        self._experimental_pyton_types = experimental_python_types
 
     def __iter__(self):
         return self._iterator
@@ -305,7 +306,8 @@ class Cursor(object):
 
         # Send prepare statement. Copy the _request object to avoid poluting the
         # one that is going to be used to execute the actual operation.
-        query = trino.client.TrinoQuery(copy.deepcopy(self._request), sql=sql)
+        query = trino.client.TrinoQuery(copy.deepcopy(self._request), sql=sql,
+                                        experimental_python_types=self._experimental_pyton_types)
         result = query.execute()
 
         # Iterate until the 'X-Trino-Added-Prepare' header is found or
@@ -327,7 +329,7 @@ class Cursor(object):
 
         # No need to deepcopy _request here because this is the actual request
         # operation
-        return trino.client.TrinoQuery(self._request, sql=sql)
+        return trino.client.TrinoQuery(self._request, sql=sql, experimental_python_types=self._experimental_pyton_types)
 
     def _format_prepared_param(self, param):
         """
@@ -359,12 +361,27 @@ class Cursor(object):
         if isinstance(param, bytes):
             return "X'%s'" % param.hex()
 
-        if isinstance(param, datetime.datetime):
-            datetime_str = param.strftime("%Y-%m-%d %H:%M:%S.%f %Z")
-            # strip trailing whitespace if param has no zone
-            datetime_str = datetime_str.rstrip(" ")
+        if isinstance(param, datetime.datetime) and param.tzinfo is None:
+            datetime_str = param.strftime("%Y-%m-%d %H:%M:%S.%f")
             return "TIMESTAMP '%s'" % datetime_str
 
+        if isinstance(param, datetime.datetime) and param.tzinfo is not None:
+            datetime_str = param.strftime("%Y-%m-%d %H:%M:%S.%f")
+            # named timezones
+            if hasattr(param.tzinfo, 'zone'):
+                return "TIMESTAMP '%s %s'" % (datetime_str, param.tzinfo.zone)
+            # offset-based timezones
+            return "TIMESTAMP '%s %s'" % (datetime_str, param.tzinfo.tzname(param))
+
+        # We can't calculate the offset for a time without a point in time
+        if isinstance(param, datetime.time) and param.tzinfo is None:
+            time_str = param.strftime("%H:%M:%S.%f")
+            return "TIME '%s'" % time_str
+
+        if isinstance(param, datetime.date):
+            date_str = param.strftime("%Y-%m-%d")
+            return "DATE '%s'" % date_str
+
         if isinstance(param, list):
             return "ARRAY[%s]" % ','.join(map(self._format_prepared_param, param))
 
@@ -379,6 +396,9 @@ class Cursor(object):
         if isinstance(param, uuid.UUID):
             return "UUID '%s'" % param
 
+        if isinstance(param, Decimal):
+            return "DECIMAL '%s'" % param
+
         raise trino.exceptions.NotSupportedError("Query parameter of type '%s' is not supported." % type(param))
 
     def _deallocate_prepare_statement(self, added_prepare_header, statement_name):
@@ -386,7 +406,8 @@ class Cursor(object):
 
         # Send deallocate statement. Copy the _request object to avoid poluting the
         # one that is going to be used to execute the actual operation.
-        query = trino.client.TrinoQuery(copy.deepcopy(self._request), sql=sql)
+        query = trino.client.TrinoQuery(copy.deepcopy(self._request), sql=sql,
+                                        experimental_python_types=self._experimental_pyton_types)
         result = query.execute(
             additional_http_headers={
                 constants.HEADER_PREPARED_STATEMENT: added_prepare_header
@@ -437,7 +458,8 @@ class Cursor(object):
                 self._deallocate_prepare_statement(added_prepare_header, statement_name)
 
         else:
-            self._query = trino.client.TrinoQuery(self._request, sql=operation)
+            self._query = trino.client.TrinoQuery(self._request, sql=operation,
+                                                  experimental_python_types=self._experimental_pyton_types)
             result = self._query.execute()
         self._iterator = iter(result)
         return result
@@ -456,6 +478,7 @@ class Cursor(object):
         """
 
         try:
+            assert self._iterator is not None
             return next(self._iterator)
         except StopIteration:
             return None
