@@ -18,7 +18,7 @@ from sqlalchemy.engine.default import DefaultDialect, DefaultExecutionContext
 from sqlalchemy.engine.url import URL
 
 from trino import dbapi as trino_dbapi, logging
-from trino.auth import BasicAuthentication
+from trino.auth import BasicAuthentication, JWTAuthentication
 from trino.dbapi import Cursor
 from trino.exceptions import TrinoUserError
 from trino.sqlalchemy import compiler, datatype, error
@@ -87,6 +87,10 @@ class TrinoDialect(DefaultDialect):
             kwargs["http_scheme"] = "https"
             kwargs["auth"] = BasicAuthentication(url.username, url.password)
 
+        if "access_token" in url.query:
+            kwargs["http_scheme"] = "https"
+            kwargs["auth"] = JWTAuthentication(url.query["access_token"])
+
         return args, kwargs
 
     def get_columns(self, connection: Connection, table_name: str, schema: str = None, **kw) -> List[Dict[str, Any]]:
@@ -225,16 +229,23 @@ class TrinoDialect(DefaultDialect):
         return []
 
     def get_table_comment(self, connection: Connection, table_name: str, schema: str = None, **kw) -> Dict[str, Any]:
-        properties_table = self._get_full_table(f"{table_name}$properties", schema)
-        query = f'SELECT "comment" FROM {properties_table}'
+        schema = schema or self._get_default_schema_name(connection)
+        if schema is None:
+            raise exc.NoSuchTableError("schema is required")
+        query = dedent(
+            """
+            SELECT "comment"
+            FROM "system"."metadata"."table_comments"
+            WHERE "schema_name" = :schema
+            AND "table_name" = :table_name
+        """
+        ).strip()
         try:
-            res = connection.execute(sql.text(query))
+            res = connection.execute(sql.text(query), schema=schema, table_name=table_name)
             return dict(text=res.scalar())
         except error.TrinoQueryError as e:
             if e.error_name in (
-                error.NOT_FOUND,
-                error.COLUMN_NOT_FOUND,
-                error.TABLE_NOT_FOUND,
+                error.PERMISSION_DENIED,
             ):
                 return dict(text=None)
             raise
